Package: NanoMethViz
Type: Package
Title: Visualise methylation data from Oxford Nanopore sequencing
Version: 3.7.0
Authors@R: c(
    person("Shian", "Su", email = "su.s@wehi.edu.au", role = c("cre", "aut")))
Description: NanoMethViz is a toolkit for visualising methylation data from
    Oxford Nanopore sequencing. It can be used to explore methylation patterns
    from reads derived from Oxford Nanopore direct DNA sequencing with
    methylation called by callers including nanopolish, f5c and megalodon. The
    plots in this package allow the visualisation of methylation profiles
    aggregated over experimental groups and across classes of genomic features.
biocViews: Software, LongRead, Visualization, DifferentialMethylation,
        DNAMethylation, Epigenetics, DataImport
URL: https://github.com/shians/NanoMethViz,
        https://shians.github.io/NanoMethViz/
BugReports: https://github.com/Shians/NanoMethViz/issues
Depends: R (>= 4.0.0), methods, ggplot2 (>= 3.4.0)
Imports: cpp11 (>= 0.2.5), readr, cli, S4Vectors, SummarizedExperiment,
        BiocSingular, bsseq, forcats, assertthat, AnnotationDbi, Rcpp,
        dplyr, dbscan, e1071, fs, GenomicRanges, Biostrings, ggrastr,
        glue, graphics, IRanges, limma (>= 3.44.0), patchwork, purrr,
        rlang, R.utils, Rsamtools, scales (>= 1.2.0), stats, stringr,
        tibble, tidyr, utils, withr
Suggests: BiocStyle, DSS, Mus.musculus (>= 1.3.1), Homo.sapiens (>=
        1.3.1), org.Hs.eg.db, TxDb.Hsapiens.UCSC.hg19.knownGene,
        TxDb.Hsapiens.UCSC.hg38.knownGene, org.Mm.eg.db,
        TxDb.Mmusculus.UCSC.mm10.knownGene,
        TxDb.Mmusculus.UCSC.mm39.refGene, knitr, rmarkdown,
        rtracklayer, testthat (>= 3.0.0), covr
LinkingTo: Rcpp
License: Apache License (>= 2.0)
SystemRequirements: C++20
VignetteBuilder: knitr
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.2
Config/testthat/parallel: true
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/NanoMethViz
git_branch: devel
git_last_commit: 4349b36
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: yes
Packaged: 2025-11-04 23:55:01 UTC; biocbuild
Author: Shian Su [cre, aut]
Maintainer: Shian Su <su.s@wehi.edu.au>
