# Generated by roxygen2: do not edit by hand

export("exons<-")
export("methy<-")
export("mod_code<-")
export("samples<-")
export(ModBamFiles)
export(ModBamResult)
export(NanoMethResult)
export(bsseq_to_edger)
export(bsseq_to_log_methy_ratio)
export(cluster_regions)
export(create_tabix_file)
export(exons)
export(exons_to_genes)
export(filter_methy)
export(get_cgi_grcm39)
export(get_cgi_hg19)
export(get_cgi_hg38)
export(get_cgi_mm10)
export(get_cgi_t2t)
export(get_example_exons_mus_musculus)
export(get_exons_grcm39)
export(get_exons_hg19)
export(get_exons_hg38)
export(get_exons_homo_sapiens)
export(get_exons_mm10)
export(get_exons_mus_musculus)
export(get_exons_t2t)
export(load_example_modbamresult)
export(load_example_nanomethresult)
export(methy)
export(methy_col_names)
export(methy_to_bsseq)
export(methy_to_edger)
export(mod_code)
export(modbam_to_tabix)
export(plot_agg_genes)
export(plot_agg_regions)
export(plot_gene)
export(plot_gene_heatmap)
export(plot_grange)
export(plot_grange_heatmap)
export(plot_mds)
export(plot_pca)
export(plot_region)
export(plot_region_heatmap)
export(plot_violin)
export(query_exons_gene_id)
export(query_exons_region)
export(query_exons_symbol)
export(query_methy)
export(region_methy_stats)
export(samples)
export(sigmoid)
exportClasses(ModBamFiles)
exportClasses(ModBamResult)
exportClasses(NanoMethResult)
exportMethods("exons<-")
exportMethods("methy<-")
exportMethods("mod_code<-")
exportMethods("samples<-")
exportMethods(exons)
exportMethods(methy)
exportMethods(mod_code)
exportMethods(plot_gene)
exportMethods(plot_gene_heatmap)
exportMethods(plot_region)
exportMethods(plot_region_heatmap)
exportMethods(samples)
exportMethods(show)
import(Rcpp)
import(assertthat)
import(cpp11)
import(dbscan)
import(dplyr)
import(fs)
import(patchwork)
import(tidyr)
importFrom(BiocSingular,runPCA)
importFrom(Biostrings,width)
importFrom(IRanges,IRanges)
importFrom(R.utils,gunzip)
importFrom(Rsamtools,TabixFile)
importFrom(Rsamtools,scanTabix)
importFrom(S4Vectors,DataFrame)
importFrom(SummarizedExperiment,colData)
importFrom(assertthat,assert_that)
importFrom(assertthat,is.dir)
importFrom(assertthat,is.readable)
importFrom(assertthat,is.string)
importFrom(assertthat,is.writeable)
importFrom(bsseq,BSseq)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(e1071,sigmoid)
importFrom(forcats,fct_reorder)
importFrom(ggplot2,aes)
importFrom(ggplot2,draw_key_point)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,rel)
importFrom(ggplot2,theme_void)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylim)
importFrom(ggrastr,rasterise)
importFrom(glue,glue)
importFrom(graphics,hist)
importFrom(limma,plotMDS)
importFrom(methods,.valueClassTest)
importFrom(methods,is)
importFrom(methods,new)
importFrom(purrr,map)
importFrom(purrr,map_lgl)
importFrom(readr,col_character)
importFrom(readr,col_double)
importFrom(readr,col_integer)
importFrom(readr,col_logical)
importFrom(readr,cols)
importFrom(readr,local_edition)
importFrom(readr,read_tsv)
importFrom(rlang,.data)
importFrom(scales,cut_si)
importFrom(scales,label_number)
importFrom(stats,approxfun)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stringr,str_extract)
importFrom(tibble,add_column)
importFrom(tibble,as_tibble)
importFrom(tibble,rownames_to_column)
importFrom(tibble,tibble)
importFrom(tidyr,unnest)
importFrom(utils,installed.packages)
importFrom(utils,packageVersion)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(withr,defer)
useDynLib(NanoMethViz)
