# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

convert_methy_to_dss_cpp <- function(input, output_dir) {
    .Call('_NanoMethViz_convert_methy_to_dss_cpp', PACKAGE = 'NanoMethViz', input, output_dir)
}

count_cg_cpp <- function(str) {
    .Call('_NanoMethViz_count_cg_cpp', PACKAGE = 'NanoMethViz', str)
}

get_char_pos_cpp <- function(x, c) {
    .Call('_NanoMethViz_get_char_pos_cpp', PACKAGE = 'NanoMethViz', x, c)
}

cigar_tokeniser_cpp <- function(x) {
    .Call('_NanoMethViz_cigar_tokeniser_cpp', PACKAGE = 'NanoMethViz', x)
}

get_coord_map_cpp <- function(cigar) {
    .Call('_NanoMethViz_get_coord_map_cpp', PACKAGE = 'NanoMethViz', cigar)
}

mod_tokeniser_cpp <- function(string, scores) {
    .Call('_NanoMethViz_mod_tokeniser_cpp', PACKAGE = 'NanoMethViz', string, scores)
}

parse_bam_cpp <- function(seq, cigar, mm_string, ml_string, map_pos, strand, mod_code) {
    .Call('_NanoMethViz_parse_bam_cpp', PACKAGE = 'NanoMethViz', seq, cigar, mm_string, ml_string, map_pos, strand, mod_code)
}

parse_bam_list_cpp <- function(seq, cigar, mm_string, ml_string, map_pos, strand, mod_code) {
    .Call('_NanoMethViz_parse_bam_list_cpp', PACKAGE = 'NanoMethViz', seq, cigar, mm_string, ml_string, map_pos, strand, mod_code)
}

