% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{samples}
\alias{samples}
\title{Get sample annotation}
\usage{
samples(object)
}
\description{
Get sample annotation
}
\keyword{internal}
