% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{mod_code<-}
\alias{mod_code<-}
\title{Set mod code}
\usage{
mod_code(object) <- value
}
\description{
Set mod code
}
\keyword{internal}
