% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{methy<-}
\alias{methy<-}
\title{Set methylation data}
\usage{
methy(object) <- value
}
\description{
Set methylation data
}
\keyword{internal}
