% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{exons}
\alias{exons}
\title{Get exon annotation}
\usage{
exons(object)
}
\description{
Get exon annotation
}
\keyword{internal}
