% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{samples<-}
\alias{samples<-}
\title{Set sample annotation}
\usage{
samples(object) <- value
}
\description{
Set sample annotation
}
\keyword{internal}
