% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AllGenerics.R
\name{exons<-}
\alias{exons<-}
\title{Set exon annotation}
\usage{
exons(object) <- value
}
\description{
Set exon annotation
}
\keyword{internal}
